
const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const conn = require(`${libs}/db/monet`);

 // Promise that executes an SQL query with optional parameters
 // ```
 // Examples:
 //   Query with no parameters:
 //     execSqlQuery('SELECT * FROM people');
 //   Query with one parameter:
 //     execSqlQuery('SELECT name, age FROM people WHERE id = ?', [1]);
 //   Query with more than one parameter:
 //     execSqlQuery('SELECT name, age FROM people WHERE city = ? AND age > ?', ['São Paulo', 35]);
 // ```
function execSqlQuery(sqlQuery, sqlQueryParams = []) {
    log.debug(`Executing SQL query '${sqlQuery}' with params '${sqlQueryParams}'`);
    return new Promise((resolve, reject) => {
        // Prepare statement
        conn.prepare(sqlQuery, true).then(
            (dbQuery) => {
                // Execute query
                dbQuery.exec(sqlQueryParams).then(
                    // Success
                    (dbResult) => {
                        log.debug(`Query result: ${dbResult.data}`);
                        log.debug(dbResult.data);
                        resolve(dbResult.data);
                    },
                    // Error
                    (dbError) => {
                        log.error(`SQL query execution error: ${dbError.message}`);
                        reject(new Error(dbError.message));
                    }
                );
                // Release resources allocated for prepared statement
                conn.release();
            }
        );
    });
}

module.exports = execSqlQuery;
